package com.logicbig.example;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;

@Component
public class EmployeeClientBean {

    @Autowired
    private DataSource dataSource;

    private NamedParameterJdbcTemplate jdbcTemplate;

    @PostConstruct
    private void postConstruct() {
        jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        saveEmployees();
    }

    public void run() {
        List<String> departments = Arrays.asList("Sale", "Account", "Admin");

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("deptParamName", departments);

        List<Employee> list = jdbcTemplate.query("SELECT * FROM Employee WHERE Dept IN (:deptParamName)",
                parameters, new RowMapper<Employee>() {
                    @Override
                    public Employee mapRow(ResultSet resultSet, int i) throws SQLException {
                        return toEmployee(resultSet);
                    }
                });

        list.stream().forEach(System.out::println);
    }

    private void saveEmployees() {
        List<Employee> list = Arrays.asList(
                Employee.create("Jim", "IT"),
                Employee.create("Sara", "Sale"),
                Employee.create("Tom", "Admin"),
                Employee.create("Diana", "IT"),
                Employee.create("Tina", "Sale"),
                Employee.create("Joe", "Account"),
                Employee.create("Lara", "IT"),
                Employee.create("Charlie", "Sale")
        );

        SimpleJdbcInsert simpleJdbcInsert = new SimpleJdbcInsert(dataSource)
                .withTableName("EMPLOYEE")
                .usingGeneratedKeyColumns("id");
        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(list.toArray());
        int[] ints = simpleJdbcInsert.executeBatch(batch);
    }

    private Employee toEmployee(ResultSet resultSet) throws SQLException {
        Employee employee = new Employee();
        employee.setId(resultSet.getLong("ID"));
        employee.setName(resultSet.getString("NAME"));
        employee.setDept(resultSet.getString("DEPT"));
        return employee;
    }
}